package com.zhengyiyu.obj;

import java.util.HashMap;

import com.zhengyiyu.util.RegionUtil;

/**
 * @author yiyu
 * The base class for all biology sequence object we will use, like Gene, RNA, DNA, promoter, enhancer, or even enriched region
 */

public class Sequence implements Comparable<Sequence> {
	
	/**
	 * should like chr1, chr2, ..., chrX, chrY, chrM
	 */
	private String chromosome;
	
	/**
	 * the start position of the seq in the chromosome, generally we count that position as in the sequence
	 */
	private int startPosition;
	
	/**
	 * the end position of the seq in the chromosome, generally we count that position as in the sequence too, so the length of the seq should be 
	 */
	private int endPosition;
	
	/**
	 * the real acgt or acgu of the sequence, most of the time, we do not need to store this content
	 */
	private String content;
	
	/**
	 * the DNA has two strand
	 */
	private Strand strand;
	
	/**
	 * this hashmap is just used for store related information of the sequence
	 */
	private HashMap<String, Object> info;
	
	/**
	 * @param chromosome
	 * @param startPosition
	 * @param endPosition
	 */
	public Sequence(String chromosome, int startPosition, int endPosition) {
		this(chromosome, startPosition, endPosition, "");
	}
	
	public Sequence(Sequence seq) {
		this(seq.getChromosome(), seq.getStartPosition(), seq.getEndPosition(), "");
	}

	/**
	 * @param chromosome
	 * @param startPosition
	 * @param endPosition
	 * @param content
	 * 
	 * if we need to contain the content in the obj, we may use it
	 * should be the mostly used constructor, startPosition will start at 1;
	 */
	public Sequence(String chromosome, int startPosition, int endPosition, String content) {
		super();
		this.chromosome = chromosome;
		
		if (startPosition <= 0) {
			startPosition = 1;
		}
		
		this.startPosition = startPosition;
		this.endPosition = endPosition;
		
		this.content = content;
		
		this.info = new HashMap<String, Object>();
	}

	/**
	 * @return the chromosome
	 */
	public String getChromosome() {
		return chromosome;
	}

	/**
	 * @param chromosome the chromosome to set
	 */
	public void setChromosome(String chromosome) {
		this.chromosome = chromosome;
	}

	/**
	 * @return the startPosition
	 */
	public int getStartPosition() {
		return startPosition;
	}
	
	/**
	 * @return the startPosition
	 */
	public int get5PrimeStartPosition() {
		return this.strand == Strand.Plus ? startPosition : endPosition;
	}
	
	public int get5PrimeEndPosition() {
		return this.strand == Strand.Plus ? endPosition : startPosition;
	}

	/**
	 * @param startPosition the startPosition to set
	 */
	public void setStartPosition(int startPosition) {
		this.startPosition = startPosition;
	}

	/**
	 * @return the endPosition
	 */
	public int getEndPosition() {
		return endPosition;
	}

	/**
	 * @param endPosition the endPosition to set
	 */
	public void setEndPosition(int endPosition) {
		this.endPosition = endPosition;
	}

	/**
	 * @return the content
	 */
	public String getContent() {
		return content;
	}

	/**
	 * @param content the content to set
	 */
	public void setContent(String content) {
		this.content = content;
	}

	/**
	 * @return the info
	 */
	public HashMap<String, Object> getInfo() {
		return info;
	}

	/**
	 * @param info the info to set
	 */
	public void setInfo(HashMap<String, Object> info) {
		this.info = info;
	}
	
	/**
	 * @return the strand
	 */
	public Strand getStrand() {
		return strand;
	}

	/**
	 * @param strand the strand to set
	 */
	public void setStrand(Strand strand) {
		this.strand = strand;
	}

	/**
	 * @param start
	 * @param end
	 * @return
	 * 
	 * compare whether the given sequence (assume that they are in the same chromosome) is overlapped with current sequence
	 */
	public boolean isOverlapWithBody(int start, int end) {
		return RegionUtil.isOverlap(this.getStartPosition(), this.getEndPosition(), start, end);
	}

	@Override
	public int compareTo(Sequence o) {
		if (o.chromosome.equals(this.chromosome)) {
			return this.startPosition - o.startPosition;
		} else {
			String thisChr = this.chromosome.substring(3);
			String oChr = o.chromosome.substring(3);
			
			// special handler for human
			if (thisChr.equals("X")) {
				thisChr = "23";
			} else if (thisChr.equals("Y")) {
				thisChr = "24";
			}
			
			if (oChr.equals("X")) {
				oChr = "23";
			} else if (oChr.equals("Y")) {
				oChr = "24";
			}
			return Integer.parseInt(thisChr) - Integer.parseInt(oChr);
		}
	}
	
	@Override
	public String toString() {
		return this.chromosome + ":" + this.startPosition + "-" + this.endPosition;
	}
	
	public int length() {
		return this.endPosition - this.startPosition + 1;
	}
	
	public int dyad() {
		return (this.startPosition + this.endPosition) / 2;
	}
	
	public String toCompactString() {
		return this.chromosome + ":" + this.startPosition + "-" + this.endPosition;
	}
	
}
